#!/usr/bin/env -S deno run --allow-all

import { join } from "https://deno.land/std@0.224.0/path/mod.ts"
import { read_conf } from "./lib/conf.ts"
import { Debouncer } from "./lib/debounce.ts"
import { fetch_repo, push_repo } from "./lib/repo.ts"
import { log } from "./lib/utils.ts"

await main()
async function main() {
	const config = await read_conf()
	await fetch_repo(config.repo_path, config.remote_url)
	const debouncer = new Debouncer(200)
	const repo_file = join(config.repo_path, "todo")
	log("Watching file", repo_file)
	for await (const _ of Deno.watchFs(repo_file)) {
		if (debouncer.should_skip()) continue
		log("Commiting event " + Date.now())
		await fetch_repo(config.repo_path, config.remote_url)
		await push_repo(config.repo_path)
	}
}
